mod\_cache\_disk

Disk based storage module for the HTTP caching filter.

Extension

mod\_cache\_disk.c

cache\_disk\_module

mod\_cache\_disk

implements a disk based storage manager for

mod\_cache

.

The headers and bodies of cached responses are stored separately on
disk, in a directory structure derived from the md5 hash of the cached
URL.

Multiple content negotiated responses can be stored concurrently,
however the caching of partial content is not yet supported by this
module.

Atomic cache updates to both header and body files are achieved without
the need for locking by storing the device and inode numbers of the body
file within the header file. This has the side effect that cache entries
manually moved into the cache will be ignored.

The

htcacheclean

tool is provided to list cached URLs, remove cached URLs, or to maintain
the size of the disk cache within size and/or inode limits. The tool can
be run on demand, or can be daemonized to offer continuous monitoring of
directory sizes.

> **Note**
>
> mod\_cache\_disk
>
> requires the services of
>
> mod\_cache
>
> , which must be loaded before mod\_cache\_disk.

> **Note**
>
> mod\_cache\_disk
>
> uses the sendfile feature to serve files from the cache when supported
> by the platform, and when enabled with
>
> EnableSendfile
>
> . However, per-directory and .htaccess configuration of
>
> EnableSendfile
>
> are ignored by
>
> mod\_cache\_disk
>
> as the corresponding settings are not available to the module when a
> request is being served from the cache.

mod\_cache

mod\_cache\_socache

Caching Guide

CacheRoot

The directory root under which cache files are stored

CacheRoot

directory

server config

virtual host

The

CacheRoot

directive defines the name of the directory on the disk to contain cache
files. If the

mod\_cache\_disk

module has been loaded or compiled in to the Apache server, this
directive

must

be defined. Failing to provide a value for

CacheRoot

will result in a configuration file processing error. The

CacheDirLevels

and

CacheDirLength

directives define the structure of the directories under the specified
root directory.

CacheRoot c:/cacheroot

CacheDirLevels

The number of levels of subdirectories in the cache.

CacheDirLevels

levels

CacheDirLevels 2

server config

virtual host

The

CacheDirLevels

directive sets the number of subdirectory levels in the cache. Cached
data will be saved this many directory levels below the

CacheRoot

directory.

A high value for

CacheDirLevels

combined with a low value for

CacheDirLength

will result in a relatively deep hierarchy, with a small number of
subdirectories at each level.

> **Note**
>
> The result of
>
> CacheDirLevels
>
> \*
>
> CacheDirLength
>
> must not be higher than 20.

CacheDirLength

The number of characters in subdirectory names

CacheDirLength

length

CacheDirLength 2

server config

virtual host

The

CacheDirLength

directive sets the number of characters for each subdirectory name in
the cache hierarchy. It can be used in conjunction with

CacheDirLevels

to determine the approximate structure of your cache hierarchy.

A high value for

CacheDirLength

combined with a low value for

CacheDirLevels

will result in a relatively flat hierarchy, with a large number of
subdirectories at each level.

> **Note**
>
> The result of
>
> CacheDirLevels
>
> \*
>
> CacheDirLength
>
> must not be higher than 20.

CacheMinFileSize

The minimum size (in bytes) of a document to be placed in the cache

CacheMinFileSize

bytes

CacheMinFileSize 1

server config

virtual host

directory

.htaccess

The

CacheMinFileSize

directive sets the minimum size, in bytes, for a document to be
considered for storage in the cache.

CacheMinFileSize 64

CacheMaxFileSize

The maximum size (in bytes) of a document to be placed in the cache

CacheMaxFileSize

bytes

CacheMaxFileSize 1000000

server config

virtual host

directory

.htaccess

The

CacheMaxFileSize

directive sets the maximum size, in bytes, for a document to be
considered for storage in the cache.

CacheMaxFileSize 64000

CacheReadSize

The minimum size (in bytes) of the document to read and be cached before
sending the data downstream

CacheReadSize

bytes

CacheReadSize 0

server config

virtual host

directory

.htaccess

The

CacheReadSize

directive sets the minimum amount of data, in bytes, to be read from the
backend before the data is sent to the client. The default of zero
causes all data read of any size to be passed downstream to the client
immediately as it arrives. Setting this to a higher value causes the
disk cache to buffer at least this amount before sending the result to
the client. This can improve performance when caching content from a
reverse proxy.

This directive only takes effect when the data is being saved to the
cache, as opposed to data being served from the cache.

CacheReadSize 102400

CacheReadTime

The minimum time (in milliseconds) that should elapse while reading
before data is sent downstream

CacheReadTime

milliseconds

CacheReadTime 0

server config

virtual host

directory

.htaccess

The

CacheReadTime

directive sets the minimum amount of elapsed time that should pass
before making an attempt to send data downstream to the client. During
the time period, data will be buffered before sending the result to the
client. This can improve performance when caching content from a reverse
proxy.

The default of zero disables this option.

This directive only takes effect when the data is being saved to the
cache, as opposed to data being served from the cache. It is recommended
that this option be used alongside the

CacheReadSize

directive to ensure that the server does not buffer excessively should
data arrive faster than expected.

CacheReadTime 1000
