mod\_charset\_lite

Specify character set translation or recoding

Extension

mod\_charset\_lite.c

charset\_lite\_module

mod\_charset\_lite

allows the server to change the character set of responses before
sending them to the client. In an EBCDIC environment, Apache always
translates HTTP protocol content (e.g. response headers) from the code
page of the Apache process locale to ISO-8859-1, but not the body of
responses. In any environment,

mod\_charset\_lite

can be used to specify that response bodies should be translated. For
example, if files are stored in EBCDIC, then

mod\_charset\_lite

can translate them to ISO-8859-1 before sending them to the client.

This module provides a small subset of configuration mechanisms
implemented by Russian Apache and its associated

mod\_charset

.

Common Problems {#problems}
===============

Invalid character set names
---------------------------

The character set name parameters of

CharsetSourceEnc

and

CharsetDefault

must be acceptable to the translation mechanism used by

APR

on the system where

mod\_charset\_lite

is deployed. These character set names are not standardized and are
usually not the same as the corresponding values used in http headers.
Currently, APR can only use iconv(3), so you can easily test your
character set names using the iconv(1) program, as follows:

iconv -f charsetsourceenc-value -t charsetdefault-value

Mismatch between character set of content and translation rules
---------------------------------------------------------------

If the translation rules don\'t make sense for the content, translation
can fail in various ways, including:

The translation mechanism may return a bad return code, and the
connection will be aborted.

The translation mechanism may silently place special characters (e.g.,
question marks) in the output buffer when it cannot translate the input
buffer.

CharsetSourceEnc

Source charset of files

CharsetSourceEnc

charset

server config

virtual host

directory

.htaccess

FileInfo

The

CharsetSourceEnc

directive specifies the source charset of files in the associated
container.

The value of the

charset

argument must be accepted as a valid character set name by the character
set support in

APR

. Generally, this means that it must be supported by iconv.

\<Directory \"/export/home/trawick/apacheinst/htdocs/convert\"\>
CharsetSourceEnc UTF-16BE CharsetDefault ISO-8859-1 \</Directory\>

The character set names in this example work with the iconv translation
support in Solaris 8.

> **Note**
>
> Specifying the same charset for both
>
> CharsetSourceEnc
>
> and
>
> CharsetDefault
>
> disables translation. The charset need not match the charset of the
> response, but it must be a valid charset on the system.

CharsetDefault

Charset to translate into

CharsetDefault

charset

server config

virtual host

directory

.htaccess

FileInfo

The

CharsetDefault

directive specifies the charset that content in the associated container
should be translated to.

The value of the

charset

argument must be accepted as a valid character set name by the character
set support in

APR

. Generally, this means that it must be supported by iconv.

\<Directory \"/export/home/trawick/apacheinst/htdocs/convert\"\>
CharsetSourceEnc UTF-16BE CharsetDefault ISO-8859-1 \</Directory\>

> **Note**
>
> Specifying the same charset for both
>
> CharsetSourceEnc
>
> and
>
> CharsetDefault
>
> disables translation. The charset need not match the charset of the
> response, but it must be a valid charset on the system.

CharsetOptions

Configures charset translation behavior

CharsetOptions

option

\[

option

\] \...

CharsetOptions ImplicitAdd

server config

virtual host

directory

.htaccess

FileInfo

The

CharsetOptions

directive configures certain behaviors of

mod\_charset\_lite

.

Option

can be one of

ImplicitAdd \| NoImplicitAdd

The

ImplicitAdd

keyword specifies that

mod\_charset\_lite

should implicitly insert its filter when the configuration specifies
that the character set of content should be translated. If the filter
chain is explicitly configured using the

AddOutputFilter

directive,

NoImplicitAdd

should be specified so that

mod\_charset\_lite

doesn\'t add its filter.

TranslateAllMimeTypes \| NoTranslateAllMimeTypes

Normally,

mod\_charset\_lite

will only perform translation on a small subset of possible mimetypes.
When the

TranslateAllMimeTypes

keyword is specified for a given configuration section, translation is
performed without regard for mimetype.
