mod\_proxy\_scgi

SCGI gateway module for

mod\_proxy

Extension

mod\_proxy\_scgi.c

proxy\_scgi\_module

This module

requires

the service of

mod\_proxy

. It provides support for the

SCGI protocol, version 1

.

Thus, in order to get the ability of handling the SCGI protocol,

mod\_proxy

and

mod\_proxy\_scgi

have to be present in the server.

> **Note**
>
> Do not enable proxying until you have
>
> secured your server
>
> . Open proxy servers are dangerous both to your network and to the
> Internet at large.

mod\_proxy

mod\_proxy\_balancer

Examples
========

Remember, in order to make the following examples work, you have to
enable

mod\_proxy

and

mod\_proxy\_scgi

.

ProxyPass \"/scgi-bin/\" \"scgi://localhost:4000/\"

The balanced gateway needs

mod\_proxy\_balancer

and at least one load balancer algorithm module, such as

mod\_lbmethod\_byrequests

, in addition to the proxy modules listed above.

mod\_lbmethod\_byrequests

is the default, and will be used for this example configuration.

ProxyPass \"/scgi-bin/\" \"balancer://somecluster/\" \<Proxy
balancer://somecluster\> BalancerMember scgi://localhost:4000
BalancerMember scgi://localhost:4001 \</Proxy\>

Environment Variables {#env}
=====================

In addition to the configuration directives that control the behaviour
of

mod\_proxy

, an

environment variable

may also control the SCGI protocol provider:

proxy-scgi-pathinfo

By default

mod\_proxy\_scgi

will neither create nor export the

PATH\_INFO

environment variable. This allows the backend SCGI server to correctly
determine

SCRIPT\_NAME

and

Script-URI

and be compliant with RFC 3875 section 3.3. If instead you need

mod\_proxy\_scgi

to generate a \"best guess\" for

PATH\_INFO

, set this env-var. The variable must be set before

SetEnv

is effective.

SetEnvIf

can be used instead:

SetEnvIf Request\_URI . proxy-scgi-pathinfo

ProxySCGISendfile

Enable evaluation of

X-Sendfile

pseudo response header

ProxySCGISendfile On\|Off\|

Headername

ProxySCGISendfile Off

server config

virtual host

directory

The

ProxySCGISendfile

directive enables the SCGI backend to let files be served directly by
the gateway. This is useful for performance purposes MDASH httpd can use

sendfile

or other optimizations, which are not possible if the file comes over
the backend socket. Additionally, the file contents are not transmitted
twice.

The

ProxySCGISendfile

argument determines the gateway behaviour:

Off

No special handling takes place.

On

The gateway looks for a backend response header called

X-Sendfile

and interprets the value as the filename to serve. The header is removed
from the final response headers. This is equivalent to

ProxySCGISendfile X-Sendfile

.

anything else

Similar to

On

, but instead of the hardcoded header name

X-Sendfile

, the argument is used as the header name.

\# Use the default header (X-Sendfile) ProxySCGISendfile On \# Use a
different header ProxySCGISendfile X-Send-Static

ProxySCGIInternalRedirect

Enable or disable internal redirect responses from the backend

ProxySCGIInternalRedirect On\|Off\|

Headername

ProxySCGIInternalRedirect On

server config

virtual host

directory

The

Headername

feature is available in Apache httpd 2.4.13 and later.

The

ProxySCGIInternalRedirect

enables the backend to internally redirect the gateway to a different
URL. This feature originates in

mod\_cgi

, which internally redirects the response if the response status is

OK

(

200

) and the response contains a

Location

(or configured alternate header) and its value starts with a slash (

/

). This value is interpreted as a new local URL that Apache httpd
internally redirects to.

mod\_proxy\_scgi

does the same as

mod\_cgi

in this regard, except that you can turn off the feature or specify the
use of a header other than

Location

.

ProxySCGIInternalRedirect Off \# Django and some other frameworks will
fully qualify \"local URLs\" \# set by the application, so an alternate
header must be used. \<Location /django-app/\> ProxySCGIInternalRedirect
X-Location \</Location\>
